/*
 ATtiny25 Tiny Temperature Logger - Transfer Data
 Version: 2.0
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 12/11/2012
 Last Modified: 10/10/2014
 
 Plug in the USB and run the a25ttl_transfer program to transfer the data to your computer.
 
 Based on V-USB example code by Christian Starkjohann and AVR ATtiny USB Tutorial at http://codeandlife.com
 
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>
#include <time.h>

// this is libusb, see http://libusb.sourceforge.net/ 
#include <usb.h>

// same as in main.c
#define USB_TRANSFER 1
#define endLoggingNumber 65278

// Modified from http://www.arduino.cc/playground/ComponentLib/Thermistor2
// Learn more at http://www.insidegadgets.com/2011/12/18/verifying-the-thermistor-function-for-temperature-reading/
// 1 Oct 2014 - Changed to use the beta parameters of thermistors
double thermistorTemp(int RawADC) {
	double Temp = log((double) (10000 / ((10240000/(double) RawADC) - 10000)));
	
	if (Temp == 0) { // Natural log of 1 is 0 which means it's 25C, but the formula below would produce 0 as the result, so manually set 25C here
		Temp = 25;
	}
	else {
		Temp = (1 / (0.00000097670833857907 * Temp)) / ((1 / (0.00029120559114735 * Temp)) - 298.15);
		Temp = Temp - 273.15; // Convert Kelvin to Celsius
	}
	
	// Method for using coefficients
	/*double Temp;
	Temp = log((double) ((10240000/RawADC) - 10000)); // 10K thermistor with 10K resistor
	Temp = 1 / (0.001138942 + (0.000232762 + (0.000000091 * Temp * Temp )) * Temp);
	Temp = Temp - 273.15; // Convert Kelvin to Celsius
	return Temp;
	*/
	
	return Temp;
}

// used to get descriptor strings for device identification 
static int usbGetDescriptorString(usb_dev_handle *dev, int index, int langid, 
                                  char *buf, int buflen) {
    char buffer[256];
    int rval, i;

	// make standard request GET_DESCRIPTOR, type string and given index 
    // (e.g. dev->iProduct)
	rval = usb_control_msg(dev, 
        USB_TYPE_STANDARD | USB_RECIP_DEVICE | USB_ENDPOINT_IN, 
        USB_REQ_GET_DESCRIPTOR, (USB_DT_STRING << 8) + index, langid, 
        buffer, sizeof(buffer), 1000);
        
    if(rval < 0) // error
		return rval;
	
    // rval should be bytes read, but buffer[0] contains the actual response size
	if((unsigned char)buffer[0] < rval)
		rval = (unsigned char)buffer[0]; // string is shorter than bytes read
	
	if(buffer[1] != USB_DT_STRING) // second byte is the data type
		return 0; // invalid return type
		
	// we're dealing with UTF-16LE here so actual chars is half of rval,
	// and index 0 doesn't count
	rval /= 2;
	
	// lossy conversion to ISO Latin1 
	for(i = 1; i < rval && i < buflen; i++) {
		if(buffer[2 * i + 1] == 0)
			buf[i-1] = buffer[2 * i];
		else
			buf[i-1] = '?'; // outside of ISO Latin1 range
	}
	buf[i-1] = 0;
	
	return i-1;
}

static usb_dev_handle * usbOpenDevice(int vendor, char *vendorName, 
                                      int product, char *productName) {
	struct usb_bus *bus;
	struct usb_device *dev;
	char devVendor[256], devProduct[256];
    
	usb_dev_handle * handle = NULL;
	
	usb_init();
	usb_find_busses();
	usb_find_devices();
	
	for(bus=usb_get_busses(); bus; bus=bus->next) {
		for(dev=bus->devices; dev; dev=dev->next) {			
			if(dev->descriptor.idVendor != vendor ||
               dev->descriptor.idProduct != product)
                continue;
                
            // we need to open the device in order to query strings 
            if(!(handle = usb_open(dev))) {
                fprintf(stderr, "Warning: cannot open USB device: %s\n",
                    usb_strerror());
                continue;
            }
            
            // get vendor name 
            if(usbGetDescriptorString(handle, dev->descriptor.iManufacturer, 0x0409, devVendor, sizeof(devVendor)) < 0) {
                fprintf(stderr, 
                    "Warning: cannot query manufacturer for device: %s\n", 
                    usb_strerror());
                usb_close(handle);
                continue;
            }
            
            // get product name 
            if(usbGetDescriptorString(handle, dev->descriptor.iProduct, 
               0x0409, devProduct, sizeof(devVendor)) < 0) {
                fprintf(stderr, 
                    "Warning: cannot query product for device: %s\n", 
                    usb_strerror());
                usb_close(handle);
                continue;
            }
            
            if(strcmp(devVendor, vendorName) == 0 && 
               strcmp(devProduct, productName) == 0)
                return handle;
            else
                usb_close(handle);
		}
	}
	
	return NULL;
}

int main(int argc, char **argv) {
	
	printf("ATtiny25 Tiny Temperature Logger: Delay Time Transfer Program v2.0\n"\
			 "Used to transfer the data from the ATtiny25 Tiny Temperature Logger\n\n");
	
	usb_dev_handle *handle = NULL;
	int nBytes = 0;
	char buffer[256];
	
	handle = usbOpenDevice(0x16C0, "insidegadgets.com", 0x05DC, "A25TTL");
	
	if (handle == NULL) {
		fprintf(stderr, "Could not find USB device!\n");
		exit(1);
	}
	
	// Get the date/time to be used for the filename
	time_t rawtime;
	struct tm* timeinfo;
	char timebuffer[80];
	
	time (&rawtime);
	timeinfo = localtime (&rawtime);
	
	strftime (timebuffer,80,"%Y-%m-%d_%H-%M-%S.txt",timeinfo);
	
	// Create a new file
	FILE *pFile = fopen(timebuffer, "wb");
	
	printf("Transferring data...\n");
	
	uint16_t readCount = 0;
	uint8_t inTransfer = 1;
	uint8_t transferredDelay = 0;
	nBytes = 1;
	while (inTransfer == 1) { 
		nBytes = usb_control_msg(handle, 
			USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, 
			USB_TRANSFER, 0, 0, (char *)buffer, sizeof(buffer), 5000);
		
		if (nBytes > 0) {
			uint8_t highByte = 0;
			uint8_t lowByte = 0;
			
			uint8_t charCount = 0;
			nBytes -= 2;
			
			while (nBytes >= 0) {
				//printf("%c", buffer[charCount]);
				
				// Print the delay time
				if (transferredDelay == 0) {
					uint8_t delayTimeSeconds = buffer[0];
					uint8_t delayTimeMinutes = buffer[1];
					uint8_t delayTimeHours = buffer[2];
					uint8_t delayTimeDays = buffer[3];
					
					char tempbuffer[50];
					char delayString[100];
					strncpy(delayString, "Delay Time: ", 13);
					
					if (delayTimeDays > 0) {
						sprintf(tempbuffer, "%d Days", delayTimeDays);
						strcat(delayString, tempbuffer);
					}
					if (delayTimeHours > 0) {
						if (delayTimeDays > 0) {
							strcat(delayString, ", ");
						}
						sprintf(tempbuffer, "%d Hours", delayTimeHours);
						strcat(delayString, tempbuffer);
					}
					if (delayTimeMinutes > 0) {
						if (delayTimeDays > 0 || delayTimeHours > 0) {
							strcat(delayString, ", ");
						}
						sprintf(tempbuffer, "%d Minutes", delayTimeMinutes);
						strcat(delayString, tempbuffer);
					}
					if (delayTimeSeconds > 0) {
						if (delayTimeDays > 0 || delayTimeHours > 0 || delayTimeMinutes > 0) {
							strcat(delayString, ", ");
						}
						sprintf(tempbuffer, "%d Seconds", delayTimeSeconds);
						strcat(delayString, tempbuffer);
					}
					
					strcat(delayString, "\r\n");
					printf("%s", delayString);
					fwrite(delayString, 1, strlen(delayString), pFile);
					transferredDelay = 1;
					charCount = 3; // Skip the delay time numbers
					nBytes -= 5;
				}
				else {
					if (charCount % 2 == 0) {
						highByte = buffer[charCount];
					}
					else {
						lowByte = buffer[charCount];
						//printf("%i = %i   %i = %i\n", charCount, highByte, charCount, lowByte);
						
						int combinedValue = (highByte << 8) | lowByte;
						//printf("%i\n", combinedValue);
						
						if (combinedValue == endLoggingNumber) { // End if we reach the the end logging number 
							inTransfer = 0;
							break;
						}
						
						readCount += 2;
						if (readCount >= 65530) { // End if we've read all of the EERPROM
							inTransfer = 0;
							break;
						}
						
						printf("%4.2f\n", thermistorTemp(combinedValue));
						
						char tempbuffer[20];
						sprintf(tempbuffer, "%4.2f\r\n", thermistorTemp(combinedValue));
						//sprintf(tempbuffer, "%i %i %4.2f\r\n", charCount, readCount, thermistorTemp(combinedValue));
						
						fwrite(tempbuffer, 1, strlen(tempbuffer), pFile);
					}
				}
				
				charCount++;
				nBytes--;
			}
		}
		else {
			inTransfer = 0;
		}
	}
	
	printf("Transfer complete\n");
	usb_close(handle);
	fclose(pFile);
	
	return 0;
}
