/*
 ATtiny25 Tiny Temperature Logger - Transfer Data
 Version: 2.0
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 12/11/2012
 Last Modified: 10/10/2014
 
 A reader for the ATtiny25 Tiny Temperature Logger used to extract data and re-program the delay time.
 We use an ATtiny85 with V-USB to read from the external I2C EEPROM.
 
 Remember to change your fuse bits so the ATtiny45/85 uses 16MHz PLL and divide clock by 8
 avrdude -p ATtiny85 -c usbtiny -U lfuse:w:0x61:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m
 
 */

// ATtiny45/85 Pin map
//
//             +-\/-+
// Reset PB5  1|o   |8  VCC
//       PB3  2|    |7  PB2
//       PB4  3|    |6  PB1
//       GND  4|    |5  PB0
//             +----+

#define F_CPU 16500000 // If this is changed, modify it in usbconfig.h too

#define usbminusPin PB1 // USB D-
#define TWI_SDA_PIN PB4
#define TWI_SCL_PIN PB0
#define delayTimelocation 65532
#define EEPROM_ADDR 0x50 // I2C EEPROM address

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "usbconfig.h"
#include "setup.c"

// USB transfer variables
#define USB_TRANSFER 1
#define USB_DELAYTIME 2
uint32_t delayTime = 0;
uint8_t sentDelayTime = 0;
uint16_t readCount = 0;
uint16_t endCount = delayTimelocation - 1;
volatile uint8_t usbUnusedCount = 0;

static uchar replyBuf[254];
static uchar *nextDigit;
static uchar dataReceived = 0, dataLength = 0; // for USB_DELAYTIME

// This gets called when custom control message is received
USB_PUBLIC uchar usbFunctionSetup(uchar data[8]) {
	usbRequest_t *rq = (void *)data; // cast data to correct type
	
	if (rq->bRequest == USB_TRANSFER) { // custom command is in the bRequest field
		SoftI2cMasterInit();
		usbUnusedCount = 0;
		nextDigit = &replyBuf[-1];
		
		for (int a = 0; a < 126; a++) {
			if (readCount <= endCount) {
				if (sentDelayTime == 0) {
					*++nextDigit = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation); // Seconds 
					*++nextDigit = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation+1); // Minutes
					*++nextDigit = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation+2); // Hours
					*++nextDigit = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation+3); // Days
					sentDelayTime = 1;
					readCount += 2;
					a += 2;
				}
				else {
					uint8_t highByte = soft_i2c_eeprom_read_byte(EEPROM_ADDR, readCount++);
					uint8_t lowByte = soft_i2c_eeprom_read_byte(EEPROM_ADDR, readCount++);
					*++nextDigit = highByte;
					*++nextDigit = lowByte;
				}
			}
			else {
				break;
			}
		}
		
		memcpy(++nextDigit, "\0", 1);
		usbMsgPtr = replyBuf;
		
		return sizeof(replyBuf);
	}
	else if (rq->bRequest == USB_DELAYTIME) { // Re-program the delay time
		SoftI2cMasterInit();
		usbUnusedCount = 0;
		
		dataLength = (uchar)rq->wLength.word;
		dataReceived = 0;
		
		if (dataLength > sizeof(replyBuf)) { // Limit to buffer size
			dataLength = sizeof(replyBuf);
		}
		
		return USB_NO_MSG; // usbFunctionWrite will be called now
	}
	
	return 0;
}

// This gets called when data is sent from PC to the device
USB_PUBLIC uchar usbFunctionWrite(uchar *data, uchar len) {
	uchar i;
	
	for (i = 0; dataReceived < dataLength && i < len; i++, dataReceived++) {
		replyBuf[dataReceived] = data[i];
	}
	
	uint8_t delayTimeProgramSeconds = replyBuf[0];
	uint8_t delayTimeProgramMinutes = replyBuf[1];
	uint8_t delayTimeProgramHours = replyBuf[2];
	uint8_t delayTimeProgramDays = replyBuf[3];
	
	soft_i2c_eeprom_write_byte(EEPROM_ADDR, delayTimelocation, delayTimeProgramSeconds); // Seconds
	soft_i2c_eeprom_write_byte(EEPROM_ADDR, delayTimelocation+1, delayTimeProgramMinutes); // Minutes
	soft_i2c_eeprom_write_byte(EEPROM_ADDR, delayTimelocation+2, delayTimeProgramHours); // Hours
	soft_i2c_eeprom_write_byte(EEPROM_ADDR, delayTimelocation+3, delayTimeProgramDays); // Days
	
	return (dataReceived == dataLength); // 1 if we received it all, 0 if not
}

int main (void) {
	setup();
	
	while(1) {
		startUSB();
		
		while (1) {
			usbPoll(); // Keep polling USB
			
			// Check if it's been 4 seconds since the last USB poll, then reset SDA/SCL pins and variables
			if (usbUnusedCount >= 20) {
				scl_sda_low();
				readCount = 0;
				sentDelayTime = 0;
				usbUnusedCount = 0;
				
				// Reset timer
				TCCR1 = 0;
				TCNT1 = 0;
				TCCR1 |= (1<<CS13) | (1<<CS12) | (1<<CS11) | (1<<CS10);
			}
		}
	}
	
	return 0;
}

ISR(TIM1_OVF_vect) { 
	usbUnusedCount++;
}
