/*
 ATtiny25 Tiny Temperature Logger (A25TTL) - works with ATtiny13 too
 Version: 2.0
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 12/11/2012
 Last Modified: 21/09/2014
 
 Used to log temperature every x seconds/minutes/hours which is saved to an external I2C EEPROM.
 You can transfer the data via USB to your computer by connecting this to the A25TTL Reader with the 
 a25ttl_transfer program and program the delay time by running the a25ttl_programdelay program.
  
 */

// ATtiny13/25 Pin map
//
//             +-\/-+
// Reset PB5  1|o   |8  VCC
//       PB3  2|    |7  PB2
//       PB4  3|    |6  PB1
//       GND  4|    |5  PB0
//             +----+

#define F_CPU 1000000 // 1 MHz clock

#define tempPin PB4 // Output
#define TWI_SDA_PIN PB1
#define TWI_SCL_PIN PB2
#define delayTimelocation 65532 // 4 bytes less than the 512Kbit EEPROM size
#define EEPROM_ADDR 0x50 // I2C EEPROM address

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include <string.h>
#include "setup.c"

uint8_t analogPin = 3; // For reading thermistor
uint16_t dataCount = 0;
uint8_t delayCount = 0;

int main (void) {
	setup();
	
	// Read delay time
	uint8_t delaySeconds = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation);
	uint8_t delayMinutes = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation+1);
	uint8_t delayHours = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation+2);
	uint8_t delayDays = soft_i2c_eeprom_read_byte(EEPROM_ADDR, delayTimelocation+3);
	if (delaySeconds == blankEeprom2byte) { // Unprogrammed delay time?
		delaySeconds = 0;
		delayMinutes = 5; // Set 5 minutes as the default delay time
		delayHours = 0;
		delayDays = 0;
	}
	
	while(1) {
		// Log Temperature
		PORTB |= (1<<tempPin); // Turn on thermistor
		_delay_us(50); // Wait a little while before reading thermistor
		
		int calculatedValue = analogRead(analogPin); // Get the reading
		PORTB &= ~(1<<tempPin); // Turn off
		
		// Write to EEPROM (don't write to where the delay time is stored)
		if (dataCount <= delayTimelocation - 1) {
			soft_i2c_eeprom_write_byte(EEPROM_ADDR, dataCount, (calculatedValue >> 8));
			dataCount++;
			soft_i2c_eeprom_write_byte(EEPROM_ADDR, dataCount, (calculatedValue & 0xFF));
			dataCount++;
			if (dataCount <= delayTimelocation - 1) {
				soft_i2c_eeprom_write_byte(EEPROM_ADDR, dataCount, 254);
				soft_i2c_eeprom_write_byte(EEPROM_ADDR, (dataCount+1), 254);
			}
		}
		else {
			// Sleep forever since the EEPROM is full
			system_sleep();
		}
		
		// Wait the delay time
		delayCount = delaySeconds;
		while (delayCount != 0) { // Seconds
			watchdogSleep(T1S);
			delayCount--;
		}
		
		delayCount = delayMinutes;
		while (delayCount != 0) { // Minutes
			for (uint8_t waitCounter = 0; waitCounter < 7; waitCounter++) {
				watchdogSleep(T8S);
			}
			watchdogSleep(T4S);
			delayCount--;
		}
		
		delayCount = delayHours;
		while (delayCount != 0) { // Hours
			for (uint16_t waitCounter = 0; waitCounter < 450; waitCounter++) {
				watchdogSleep(T8S);
			}
			delayCount--;
		}
		
		delayCount = delayDays;
		while (delayCount != 0) { // Days
			for (uint16_t waitCounter = 0; waitCounter < 10800; waitCounter++) {
				watchdogSleep(T8S);
			}
			delayCount--;
		}
	}
	
	return 0;
}

ISR(WDT_vect) { }

ISR(ADC_vect) { }
